import importlib
import os

from curia.utils.s3 import AWSConfig


def execute() -> None:
    """
    Executes an arbitrary python ContainerTaskDefinition passed through the command line
    :return:
    """

    if os.environ.get("TASK_NAME", None) is None:
        raise ValueError("TASK_NAME envvar is required")
    if os.environ.get("TASK_MODULE", None) is None:
        raise ValueError("TASK_MODULE is required")

    # import the task
    task_module = importlib.import_module(os.environ["TASK_MODULE"])
    task = getattr(task_module, os.environ["TASK_NAME"])

    task.run(
        task_execution_id=os.environ["TASK_EXECUTION_ID"],
        api_token=os.environ["API_TOKEN"],
        api_endpoint=os.environ["API_ENDPOINT"],
        # optional parameters, mainly used for local development / testing
        aws_config=AWSConfig(
            aws_region=os.environ.get("AWS_REGION"),
            aws_endpoint_url=os.environ.get("AWS_ENDPOINT_URL"),
            aws_access_key_id=os.environ.get("AWS_ACCESS_KEY_ID"),
            aws_secret_access_key=os.environ.get("AWS_SECRET_ACCESS_KEY"),
        )
    )


if __name__ == "__main__":
    execute()



import pandas as pd

from curia.api.swagger_client import Dataset
from curia.session import Session
from curia.utils.s3 import load_parquet_files_from_s3


class PredictedModelJob:
    def __init__(self, session: Session, model_job_id: str) -> None:
        self.session = session
        self.model_job_id = model_job_id

    def predictions(self) -> pd.DataFrame:
        """Get the predictions from a model job.

        Returns:
            pd.DataFrame: The predictions.
        """
        predictions_outputs = self.session.api_instance.get_many_base_model_job_output_controller_model_job_output(
            filter=[f"modelJobId||$eq||{self.model_job_id}", "name||$starts||predictions"]
        ).data
        if len(predictions_outputs) == 0:
            raise ValueError(f"No predictions found for model job {self.model_job_id}, verify model job was run"
                             f" successfully and that the job is a prediction job.")
        predictions_output = predictions_outputs[0]

        predictions_dataset: Dataset = self.session.api_instance.get_one_base_dataset_controller_dataset(
            id=predictions_output.dataset_id
        )
        return load_parquet_files_from_s3(predictions_dataset.location)


import dataclasses
from enum import Enum


@dataclasses.dataclass(frozen=True)
class TaskDeployResourceNamingConfig:
    platform_task_prefix: str
    platform_workflow_prefix: str


@dataclasses.dataclass(frozen=True)
class ContainerTaskDeployResourceNamingConfig(TaskDeployResourceNamingConfig):
    image_prefix: str


@dataclasses.dataclass(frozen=True)
class DatabricksTaskDeployResourceNamingConfig(TaskDeployResourceNamingConfig):
    databricks_workflow_prefix: str


class TaskType(Enum):
    """
    Enum for the different types of tasks. Should match the types defined in type.setter in
    curia/api/swagger_client/models/task.py, along with an Abstract type that specifies that the task
    is not a concrete task type, but rather a base class for other task types.
    """
    ABSTRACT = "Abstract"
    DATAQUERY = "DataQuery"
    CONTAINER = "ContainerExecution"
    DATABRICKS = "DatabricksJob"


class DatabricksTaskCodeSourceType(Enum):
    """
    Enum for the different types of code sources for databricks tasks. Currently supports S3WHEEL, PRIVATE_PYPI
    """
    S3WHEEL = "S3WHEEL"
    PRIVATE_PYPI = "PRIVATE_PYPI"




# coding: utf-8

"""
    Curia Platform API

    These are the docs for the curia platform API. To test, generate an authorization token first.  # noqa: E501

    OpenAPI spec version: 3.16.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class InterventionDefinition(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'id': 'str',
        'person_set': 'PersonSet',
        'model_populations': 'list[ModelPopulation]',
        'population_id': 'str',
        'population': 'Population',
        'dataset_id': 'str',
        'dataset': 'Dataset',
        'created_at': 'datetime',
        'created_by': 'str',
        'updated_at': 'datetime',
        'archived_at': 'datetime',
        'last_updated_by': 'str',
        'version': 'float'
    }

    attribute_map = {
        'id': 'id',
        'person_set': 'personSet',
        'model_populations': 'modelPopulations',
        'population_id': 'populationId',
        'population': 'population',
        'dataset_id': 'datasetId',
        'dataset': 'dataset',
        'created_at': 'createdAt',
        'created_by': 'createdBy',
        'updated_at': 'updatedAt',
        'archived_at': 'archivedAt',
        'last_updated_by': 'lastUpdatedBy',
        'version': 'version'
    }

    def __init__(self, id=None, person_set=None, model_populations=None, population_id=None, population=None, dataset_id=None, dataset=None, created_at=None, created_by=None, updated_at=None, archived_at=None, last_updated_by=None, version=None):  # noqa: E501
        """InterventionDefinition - a model defined in Swagger"""  # noqa: E501
        self._id = None
        self._person_set = None
        self._model_populations = None
        self._population_id = None
        self._population = None
        self._dataset_id = None
        self._dataset = None
        self._created_at = None
        self._created_by = None
        self._updated_at = None
        self._archived_at = None
        self._last_updated_by = None
        self._version = None
        self.discriminator = None
        if id is not None:
            self.id = id
        if person_set is not None:
            self.person_set = person_set
        if model_populations is not None:
            self.model_populations = model_populations
        if population_id is not None:
            self.population_id = population_id
        if population is not None:
            self.population = population
        if dataset_id is not None:
            self.dataset_id = dataset_id
        if dataset is not None:
            self.dataset = dataset
        if created_at is not None:
            self.created_at = created_at
        if created_by is not None:
            self.created_by = created_by
        if updated_at is not None:
            self.updated_at = updated_at
        if archived_at is not None:
            self.archived_at = archived_at
        if last_updated_by is not None:
            self.last_updated_by = last_updated_by
        if version is not None:
            self.version = version

    @property
    def id(self):
        """Gets the id of this InterventionDefinition.  # noqa: E501


        :return: The id of this InterventionDefinition.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this InterventionDefinition.


        :param id: The id of this InterventionDefinition.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def person_set(self):
        """Gets the person_set of this InterventionDefinition.  # noqa: E501


        :return: The person_set of this InterventionDefinition.  # noqa: E501
        :rtype: PersonSet
        """
        return self._person_set

    @person_set.setter
    def person_set(self, person_set):
        """Sets the person_set of this InterventionDefinition.


        :param person_set: The person_set of this InterventionDefinition.  # noqa: E501
        :type: PersonSet
        """

        self._person_set = person_set

    @property
    def model_populations(self):
        """Gets the model_populations of this InterventionDefinition.  # noqa: E501


        :return: The model_populations of this InterventionDefinition.  # noqa: E501
        :rtype: list[ModelPopulation]
        """
        return self._model_populations

    @model_populations.setter
    def model_populations(self, model_populations):
        """Sets the model_populations of this InterventionDefinition.


        :param model_populations: The model_populations of this InterventionDefinition.  # noqa: E501
        :type: list[ModelPopulation]
        """

        self._model_populations = model_populations

    @property
    def population_id(self):
        """Gets the population_id of this InterventionDefinition.  # noqa: E501


        :return: The population_id of this InterventionDefinition.  # noqa: E501
        :rtype: str
        """
        return self._population_id

    @population_id.setter
    def population_id(self, population_id):
        """Sets the population_id of this InterventionDefinition.


        :param population_id: The population_id of this InterventionDefinition.  # noqa: E501
        :type: str
        """

        self._population_id = population_id

    @property
    def population(self):
        """Gets the population of this InterventionDefinition.  # noqa: E501


        :return: The population of this InterventionDefinition.  # noqa: E501
        :rtype: Population
        """
        return self._population

    @population.setter
    def population(self, population):
        """Sets the population of this InterventionDefinition.


        :param population: The population of this InterventionDefinition.  # noqa: E501
        :type: Population
        """

        self._population = population

    @property
    def dataset_id(self):
        """Gets the dataset_id of this InterventionDefinition.  # noqa: E501


        :return: The dataset_id of this InterventionDefinition.  # noqa: E501
        :rtype: str
        """
        return self._dataset_id

    @dataset_id.setter
    def dataset_id(self, dataset_id):
        """Sets the dataset_id of this InterventionDefinition.


        :param dataset_id: The dataset_id of this InterventionDefinition.  # noqa: E501
        :type: str
        """

        self._dataset_id = dataset_id

    @property
    def dataset(self):
        """Gets the dataset of this InterventionDefinition.  # noqa: E501


        :return: The dataset of this InterventionDefinition.  # noqa: E501
        :rtype: Dataset
        """
        return self._dataset

    @dataset.setter
    def dataset(self, dataset):
        """Sets the dataset of this InterventionDefinition.


        :param dataset: The dataset of this InterventionDefinition.  # noqa: E501
        :type: Dataset
        """

        self._dataset = dataset

    @property
    def created_at(self):
        """Gets the created_at of this InterventionDefinition.  # noqa: E501


        :return: The created_at of this InterventionDefinition.  # noqa: E501
        :rtype: datetime
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at):
        """Sets the created_at of this InterventionDefinition.


        :param created_at: The created_at of this InterventionDefinition.  # noqa: E501
        :type: datetime
        """

        self._created_at = created_at

    @property
    def created_by(self):
        """Gets the created_by of this InterventionDefinition.  # noqa: E501


        :return: The created_by of this InterventionDefinition.  # noqa: E501
        :rtype: str
        """
        return self._created_by

    @created_by.setter
    def created_by(self, created_by):
        """Sets the created_by of this InterventionDefinition.


        :param created_by: The created_by of this InterventionDefinition.  # noqa: E501
        :type: str
        """

        self._created_by = created_by

    @property
    def updated_at(self):
        """Gets the updated_at of this InterventionDefinition.  # noqa: E501


        :return: The updated_at of this InterventionDefinition.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_at

    @updated_at.setter
    def updated_at(self, updated_at):
        """Sets the updated_at of this InterventionDefinition.


        :param updated_at: The updated_at of this InterventionDefinition.  # noqa: E501
        :type: datetime
        """

        self._updated_at = updated_at

    @property
    def archived_at(self):
        """Gets the archived_at of this InterventionDefinition.  # noqa: E501


        :return: The archived_at of this InterventionDefinition.  # noqa: E501
        :rtype: datetime
        """
        return self._archived_at

    @archived_at.setter
    def archived_at(self, archived_at):
        """Sets the archived_at of this InterventionDefinition.


        :param archived_at: The archived_at of this InterventionDefinition.  # noqa: E501
        :type: datetime
        """

        self._archived_at = archived_at

    @property
    def last_updated_by(self):
        """Gets the last_updated_by of this InterventionDefinition.  # noqa: E501


        :return: The last_updated_by of this InterventionDefinition.  # noqa: E501
        :rtype: str
        """
        return self._last_updated_by

    @last_updated_by.setter
    def last_updated_by(self, last_updated_by):
        """Sets the last_updated_by of this InterventionDefinition.


        :param last_updated_by: The last_updated_by of this InterventionDefinition.  # noqa: E501
        :type: str
        """

        self._last_updated_by = last_updated_by

    @property
    def version(self):
        """Gets the version of this InterventionDefinition.  # noqa: E501


        :return: The version of this InterventionDefinition.  # noqa: E501
        :rtype: float
        """
        return self._version

    @version.setter
    def version(self, version):
        """Sets the version of this InterventionDefinition.


        :param version: The version of this InterventionDefinition.  # noqa: E501
        :type: float
        """

        self._version = version

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(InterventionDefinition, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, InterventionDefinition):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Curia Platform API

    These are the docs for the curia platform API. To test, generate an authorization token first.  # noqa: E501

    OpenAPI spec version: 3.2.1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class CreateAnalysisJobDto(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'analysis_id': 'str',
        'type': 'str',
        'project_id': 'str',
        'execution_id': 'str',
        'status': 'str',
        'config': 'AnalysisJobConfig',
        'started_at': 'datetime',
        'ended_at': 'datetime'
    }

    attribute_map = {
        'analysis_id': 'analysisId',
        'type': 'type',
        'project_id': 'projectId',
        'execution_id': 'executionId',
        'status': 'status',
        'config': 'config',
        'started_at': 'startedAt',
        'ended_at': 'endedAt'
    }

    def __init__(self, analysis_id=None, type=None, project_id=None, execution_id=None, status=None, config=None, started_at=None, ended_at=None):  # noqa: E501
        """CreateAnalysisJobDto - a model defined in Swagger"""  # noqa: E501
        self._analysis_id = None
        self._type = None
        self._project_id = None
        self._execution_id = None
        self._status = None
        self._config = None
        self._started_at = None
        self._ended_at = None
        self.discriminator = None
        self.analysis_id = analysis_id
        self.type = type
        self.project_id = project_id
        if execution_id is not None:
            self.execution_id = execution_id
        if status is not None:
            self.status = status
        if config is not None:
            self.config = config
        if started_at is not None:
            self.started_at = started_at
        if ended_at is not None:
            self.ended_at = ended_at

    @property
    def analysis_id(self):
        """Gets the analysis_id of this CreateAnalysisJobDto.  # noqa: E501


        :return: The analysis_id of this CreateAnalysisJobDto.  # noqa: E501
        :rtype: str
        """
        return self._analysis_id

    @analysis_id.setter
    def analysis_id(self, analysis_id):
        """Sets the analysis_id of this CreateAnalysisJobDto.


        :param analysis_id: The analysis_id of this CreateAnalysisJobDto.  # noqa: E501
        :type: str
        """
        if analysis_id is None:
            raise ValueError("Invalid value for `analysis_id`, must not be `None`")  # noqa: E501

        self._analysis_id = analysis_id

    @property
    def type(self):
        """Gets the type of this CreateAnalysisJobDto.  # noqa: E501


        :return: The type of this CreateAnalysisJobDto.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this CreateAnalysisJobDto.


        :param type: The type of this CreateAnalysisJobDto.  # noqa: E501
        :type: str
        """
        if type is None:
            raise ValueError("Invalid value for `type`, must not be `None`")  # noqa: E501
        allowed_values = ["train", "predict"]  # noqa: E501
        if type not in allowed_values:
            raise ValueError(
                "Invalid value for `type` ({0}), must be one of {1}"  # noqa: E501
                .format(type, allowed_values)
            )

        self._type = type

    @property
    def project_id(self):
        """Gets the project_id of this CreateAnalysisJobDto.  # noqa: E501


        :return: The project_id of this CreateAnalysisJobDto.  # noqa: E501
        :rtype: str
        """
        return self._project_id

    @project_id.setter
    def project_id(self, project_id):
        """Sets the project_id of this CreateAnalysisJobDto.


        :param project_id: The project_id of this CreateAnalysisJobDto.  # noqa: E501
        :type: str
        """
        if project_id is None:
            raise ValueError("Invalid value for `project_id`, must not be `None`")  # noqa: E501

        self._project_id = project_id

    @property
    def execution_id(self):
        """Gets the execution_id of this CreateAnalysisJobDto.  # noqa: E501


        :return: The execution_id of this CreateAnalysisJobDto.  # noqa: E501
        :rtype: str
        """
        return self._execution_id

    @execution_id.setter
    def execution_id(self, execution_id):
        """Sets the execution_id of this CreateAnalysisJobDto.


        :param execution_id: The execution_id of this CreateAnalysisJobDto.  # noqa: E501
        :type: str
        """

        self._execution_id = execution_id

    @property
    def status(self):
        """Gets the status of this CreateAnalysisJobDto.  # noqa: E501


        :return: The status of this CreateAnalysisJobDto.  # noqa: E501
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this CreateAnalysisJobDto.


        :param status: The status of this CreateAnalysisJobDto.  # noqa: E501
        :type: str
        """

        self._status = status

    @property
    def config(self):
        """Gets the config of this CreateAnalysisJobDto.  # noqa: E501


        :return: The config of this CreateAnalysisJobDto.  # noqa: E501
        :rtype: AnalysisJobConfig
        """
        return self._config

    @config.setter
    def config(self, config):
        """Sets the config of this CreateAnalysisJobDto.


        :param config: The config of this CreateAnalysisJobDto.  # noqa: E501
        :type: AnalysisJobConfig
        """

        self._config = config

    @property
    def started_at(self):
        """Gets the started_at of this CreateAnalysisJobDto.  # noqa: E501


        :return: The started_at of this CreateAnalysisJobDto.  # noqa: E501
        :rtype: datetime
        """
        return self._started_at

    @started_at.setter
    def started_at(self, started_at):
        """Sets the started_at of this CreateAnalysisJobDto.


        :param started_at: The started_at of this CreateAnalysisJobDto.  # noqa: E501
        :type: datetime
        """

        self._started_at = started_at

    @property
    def ended_at(self):
        """Gets the ended_at of this CreateAnalysisJobDto.  # noqa: E501


        :return: The ended_at of this CreateAnalysisJobDto.  # noqa: E501
        :rtype: datetime
        """
        return self._ended_at

    @ended_at.setter
    def ended_at(self, ended_at):
        """Sets the ended_at of this CreateAnalysisJobDto.


        :param ended_at: The ended_at of this CreateAnalysisJobDto.  # noqa: E501
        :type: datetime
        """

        self._ended_at = ended_at

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(CreateAnalysisJobDto, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, CreateAnalysisJobDto):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



import functools
import json
import os
import time
import traceback
from base64 import b64encode
from datetime import datetime
from multiprocessing import Pipe
from multiprocessing.context import Process
from typing import Dict, Optional
from urllib.parse import urlparse

import moto.s3.responses
from curia.mock.s3 import _create_seed_s3_data
from moto.server import DomainDispatcherApplication, create_backend_app
from werkzeug import run_simple

from curia.utils.s3 import AWSIoContext


class MotoServerFailedException(Exception):
    pass


class ExceptionSafeProcess(Process):
    """
    A process subclass that catches exceptions and stores them in a property.
    """
    def __init__(self, *args, **kwargs):
        Process.__init__(self, *args, **kwargs)
        self._pconn, self._cconn = Pipe()
        self._exception = None

    def run(self):
        try:
            Process.run(self)
            self._cconn.send(None)
        except Exception as e:
            tb = traceback.format_exc()
            self._cconn.send((str(e), tb))

    @property
    def exception(self):
        if self._pconn.poll():
            self._exception = self._pconn.recv()
        return self._exception


def _wrap_subdomain_based_buckets(subdomain_based_buckets):
    """
    Decorator that enables moto3 to correctly identify host.docker.internal as being a path similar to localhost.
    This is necessary because the requests coming from a docker container are not recognized as localhost.
    """
    @functools.wraps(subdomain_based_buckets)
    def wrapper(self, request):
        host = request.headers.get("host", request.headers.get("Host"))
        if not host:
            host = urlparse(request.url).netloc
        if host.startswith("host.docker.internal"):
            return False
        return subdomain_based_buckets(self, request)

    return wrapper


# patch moto with decorator
moto.s3.responses.S3Response.subdomain_based_buckets = \
    _wrap_subdomain_based_buckets(moto.s3.responses.S3Response.subdomain_based_buckets)


def _patch_ecr_docker_creds(mock_ecr_docker_registry_credentials: Dict[str, str]):
    def wrapper_fn(fn):
        @functools.wraps(fn)
        def patch_fn(*args, **kwargs): # pylint: disable=unused-argument
            token_text = (f'{mock_ecr_docker_registry_credentials["username"]}:'
                          f'{mock_ecr_docker_registry_credentials["password"]}')
            return json.dumps({"authorizationData": [{
                'authorizationToken': b64encode(
                    token_text.encode("ascii")
                ).decode(),
                'expiresAt': time.mktime(datetime(2015, 1, 1).timetuple()),
                "proxyEndpoint": f"https://{mock_ecr_docker_registry_credentials['host']}",
            }]})
        return patch_fn
    return wrapper_fn


def run_moto_server(port=5000, ecr_docker_registry_credentials: Optional[Dict[str, str]] = None):
    """
    Run a moto server in a separate process.
    """
    if ecr_docker_registry_credentials:
        import moto.ecr.responses  # pylint: disable=import-outside-toplevel,redefined-outer-name
        moto.ecr.responses.ECRResponse.get_authorization_token = (
            _patch_ecr_docker_creds(ecr_docker_registry_credentials)(moto.ecr.responses.ECRResponse.get_authorization_token)
        )
    # EK:
    # Python has a notorious unsolved bug when spinning up a process that uses UrlLib in another thread. In particular,
    # system frameworks using UrlLib make calls to libdispatch to query the system configuration for network proxies.
    # libdispatch is not thread-safe, so this leads to a segfault when used in a separate process.
    # Since this is a test server that does not need any network proxies, we disable it here.
    # See these two python bug threads for more info: https://bugs.python.org/issue30385
    # https://bugs.python.org/issue39853
    os.environ["no_proxy"] = "*"
    main_app = DomainDispatcherApplication(create_backend_app)
    main_app.debug = False
    run_simple(
        hostname="127.0.0.1",
        port=port,
        application=main_app,
        threaded=False,
        use_reloader=False,
        ssl_context=None
    )


def using_moto_server(port=5000,
                      seed_s3_data: Dict[str, dict] = None,
                      ecr_docker_registry_credentials: Dict[str, str] = None) -> callable:
    """
    Decorator that starts a moto server in a separate process and kills it after the decorated function has finished.
    """
    def wrap(fn):
        @functools.wraps(fn)
        def wrapper(*args, **kwargs):
            moto_kwargs = {'port': port}
            if ecr_docker_registry_credentials:
                moto_kwargs['ecr_docker_registry_credentials'] = ecr_docker_registry_credentials
            server = ExceptionSafeProcess(
                target=run_moto_server,
                kwargs=moto_kwargs,
                daemon=True
            )
            server.start()
            try:
                with AWSIoContext(
                    aws_access_key_id="dummy",
                    aws_secret_access_key="dummy",
                    aws_region="us-east-1",
                    aws_endpoint_url=f"http://localhost:{port}/",
                ):
                    if seed_s3_data:
                        _create_seed_s3_data(seed_s3_data)
                    res = fn(*args, **kwargs)
            finally:
                exception = server.exception
                server.terminate()
                server.join()
                if exception:
                    error, tb = exception
                    raise MotoServerFailedException(
                        f"Moto server failed to start with: {error}.\n Traceback: \n#####{tb}\n#####\n\n"
                    )
            return res

        return wrapper

    return wrap


if __name__ == "__main__":
    run_moto_server()


# coding: utf-8

"""
    Curia Platform API

    These are the docs for the curia platform API. To test, generate an authorization token first.  # noqa: E501

    OpenAPI spec version: 3.16.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class ModelPopulationJoinedFeatureResponseDto(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'id': 'str',
        'column_name': 'str',
        'column_alias': 'str',
        'display_name': 'str',
        'feature_sub_category_id': 'str',
        'aggregation_type': 'str',
        'is_filter': 'bool',
        'is_trainable': 'bool',
        'is_explainable': 'bool',
        'is_categorical': 'bool',
        'category_values': 'object',
        'type': 'str',
        'last_updated_by': 'str',
        'created_by': 'str',
        'created_at': 'datetime',
        'updated_at': 'datetime',
        'archived_at': 'datetime',
        'version': 'float'
    }

    attribute_map = {
        'id': 'id',
        'column_name': 'columnName',
        'column_alias': 'columnAlias',
        'display_name': 'displayName',
        'feature_sub_category_id': 'featureSubCategoryId',
        'aggregation_type': 'aggregationType',
        'is_filter': 'isFilter',
        'is_trainable': 'isTrainable',
        'is_explainable': 'isExplainable',
        'is_categorical': 'isCategorical',
        'category_values': 'categoryValues',
        'type': 'type',
        'last_updated_by': 'lastUpdatedBy',
        'created_by': 'createdBy',
        'created_at': 'createdAt',
        'updated_at': 'updatedAt',
        'archived_at': 'archivedAt',
        'version': 'version'
    }

    def __init__(self, id=None, column_name=None, column_alias=None, display_name=None, feature_sub_category_id=None, aggregation_type=None, is_filter=None, is_trainable=None, is_explainable=None, is_categorical=None, category_values=None, type=None, last_updated_by=None, created_by=None, created_at=None, updated_at=None, archived_at=None, version=None):  # noqa: E501
        """ModelPopulationJoinedFeatureResponseDto - a model defined in Swagger"""  # noqa: E501
        self._id = None
        self._column_name = None
        self._column_alias = None
        self._display_name = None
        self._feature_sub_category_id = None
        self._aggregation_type = None
        self._is_filter = None
        self._is_trainable = None
        self._is_explainable = None
        self._is_categorical = None
        self._category_values = None
        self._type = None
        self._last_updated_by = None
        self._created_by = None
        self._created_at = None
        self._updated_at = None
        self._archived_at = None
        self._version = None
        self.discriminator = None
        if id is not None:
            self.id = id
        self.column_name = column_name
        self.column_alias = column_alias
        self.display_name = display_name
        self.feature_sub_category_id = feature_sub_category_id
        self.aggregation_type = aggregation_type
        self.is_filter = is_filter
        self.is_trainable = is_trainable
        self.is_explainable = is_explainable
        self.is_categorical = is_categorical
        if category_values is not None:
            self.category_values = category_values
        self.type = type
        if last_updated_by is not None:
            self.last_updated_by = last_updated_by
        if created_by is not None:
            self.created_by = created_by
        if created_at is not None:
            self.created_at = created_at
        if updated_at is not None:
            self.updated_at = updated_at
        if archived_at is not None:
            self.archived_at = archived_at
        if version is not None:
            self.version = version

    @property
    def id(self):
        """Gets the id of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501


        :return: The id of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this ModelPopulationJoinedFeatureResponseDto.


        :param id: The id of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def column_name(self):
        """Gets the column_name of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501


        :return: The column_name of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._column_name

    @column_name.setter
    def column_name(self, column_name):
        """Sets the column_name of this ModelPopulationJoinedFeatureResponseDto.


        :param column_name: The column_name of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501
        :type: str
        """
        if column_name is None:
            raise ValueError("Invalid value for `column_name`, must not be `None`")  # noqa: E501

        self._column_name = column_name

    @property
    def column_alias(self):
        """Gets the column_alias of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501


        :return: The column_alias of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._column_alias

    @column_alias.setter
    def column_alias(self, column_alias):
        """Sets the column_alias of this ModelPopulationJoinedFeatureResponseDto.


        :param column_alias: The column_alias of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501
        :type: str
        """
        if column_alias is None:
            raise ValueError("Invalid value for `column_alias`, must not be `None`")  # noqa: E501

        self._column_alias = column_alias

    @property
    def display_name(self):
        """Gets the display_name of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501


        :return: The display_name of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._display_name

    @display_name.setter
    def display_name(self, display_name):
        """Sets the display_name of this ModelPopulationJoinedFeatureResponseDto.


        :param display_name: The display_name of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501
        :type: str
        """
        if display_name is None:
            raise ValueError("Invalid value for `display_name`, must not be `None`")  # noqa: E501

        self._display_name = display_name

    @property
    def feature_sub_category_id(self):
        """Gets the feature_sub_category_id of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501


        :return: The feature_sub_category_id of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._feature_sub_category_id

    @feature_sub_category_id.setter
    def feature_sub_category_id(self, feature_sub_category_id):
        """Sets the feature_sub_category_id of this ModelPopulationJoinedFeatureResponseDto.


        :param feature_sub_category_id: The feature_sub_category_id of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501
        :type: str
        """
        if feature_sub_category_id is None:
            raise ValueError("Invalid value for `feature_sub_category_id`, must not be `None`")  # noqa: E501

        self._feature_sub_category_id = feature_sub_category_id

    @property
    def aggregation_type(self):
        """Gets the aggregation_type of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501


        :return: The aggregation_type of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._aggregation_type

    @aggregation_type.setter
    def aggregation_type(self, aggregation_type):
        """Sets the aggregation_type of this ModelPopulationJoinedFeatureResponseDto.


        :param aggregation_type: The aggregation_type of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501
        :type: str
        """
        if aggregation_type is None:
            raise ValueError("Invalid value for `aggregation_type`, must not be `None`")  # noqa: E501
        allowed_values = ["avg", "sum"]  # noqa: E501
        if aggregation_type not in allowed_values:
            raise ValueError(
                "Invalid value for `aggregation_type` ({0}), must be one of {1}"  # noqa: E501
                .format(aggregation_type, allowed_values)
            )

        self._aggregation_type = aggregation_type

    @property
    def is_filter(self):
        """Gets the is_filter of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501


        :return: The is_filter of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501
        :rtype: bool
        """
        return self._is_filter

    @is_filter.setter
    def is_filter(self, is_filter):
        """Sets the is_filter of this ModelPopulationJoinedFeatureResponseDto.


        :param is_filter: The is_filter of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501
        :type: bool
        """
        if is_filter is None:
            raise ValueError("Invalid value for `is_filter`, must not be `None`")  # noqa: E501

        self._is_filter = is_filter

    @property
    def is_trainable(self):
        """Gets the is_trainable of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501


        :return: The is_trainable of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501
        :rtype: bool
        """
        return self._is_trainable

    @is_trainable.setter
    def is_trainable(self, is_trainable):
        """Sets the is_trainable of this ModelPopulationJoinedFeatureResponseDto.


        :param is_trainable: The is_trainable of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501
        :type: bool
        """
        if is_trainable is None:
            raise ValueError("Invalid value for `is_trainable`, must not be `None`")  # noqa: E501

        self._is_trainable = is_trainable

    @property
    def is_explainable(self):
        """Gets the is_explainable of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501


        :return: The is_explainable of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501
        :rtype: bool
        """
        return self._is_explainable

    @is_explainable.setter
    def is_explainable(self, is_explainable):
        """Sets the is_explainable of this ModelPopulationJoinedFeatureResponseDto.


        :param is_explainable: The is_explainable of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501
        :type: bool
        """
        if is_explainable is None:
            raise ValueError("Invalid value for `is_explainable`, must not be `None`")  # noqa: E501

        self._is_explainable = is_explainable

    @property
    def is_categorical(self):
        """Gets the is_categorical of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501


        :return: The is_categorical of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501
        :rtype: bool
        """
        return self._is_categorical

    @is_categorical.setter
    def is_categorical(self, is_categorical):
        """Sets the is_categorical of this ModelPopulationJoinedFeatureResponseDto.


        :param is_categorical: The is_categorical of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501
        :type: bool
        """
        if is_categorical is None:
            raise ValueError("Invalid value for `is_categorical`, must not be `None`")  # noqa: E501

        self._is_categorical = is_categorical

    @property
    def category_values(self):
        """Gets the category_values of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501


        :return: The category_values of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501
        :rtype: object
        """
        return self._category_values

    @category_values.setter
    def category_values(self, category_values):
        """Sets the category_values of this ModelPopulationJoinedFeatureResponseDto.


        :param category_values: The category_values of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501
        :type: object
        """

        self._category_values = category_values

    @property
    def type(self):
        """Gets the type of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501


        :return: The type of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this ModelPopulationJoinedFeatureResponseDto.


        :param type: The type of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501
        :type: str
        """
        if type is None:
            raise ValueError("Invalid value for `type`, must not be `None`")  # noqa: E501
        allowed_values = ["string", "numeric"]  # noqa: E501
        if type not in allowed_values:
            raise ValueError(
                "Invalid value for `type` ({0}), must be one of {1}"  # noqa: E501
                .format(type, allowed_values)
            )

        self._type = type

    @property
    def last_updated_by(self):
        """Gets the last_updated_by of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501


        :return: The last_updated_by of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._last_updated_by

    @last_updated_by.setter
    def last_updated_by(self, last_updated_by):
        """Sets the last_updated_by of this ModelPopulationJoinedFeatureResponseDto.


        :param last_updated_by: The last_updated_by of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501
        :type: str
        """

        self._last_updated_by = last_updated_by

    @property
    def created_by(self):
        """Gets the created_by of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501


        :return: The created_by of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._created_by

    @created_by.setter
    def created_by(self, created_by):
        """Sets the created_by of this ModelPopulationJoinedFeatureResponseDto.


        :param created_by: The created_by of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501
        :type: str
        """

        self._created_by = created_by

    @property
    def created_at(self):
        """Gets the created_at of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501


        :return: The created_at of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501
        :rtype: datetime
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at):
        """Sets the created_at of this ModelPopulationJoinedFeatureResponseDto.


        :param created_at: The created_at of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501
        :type: datetime
        """

        self._created_at = created_at

    @property
    def updated_at(self):
        """Gets the updated_at of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501


        :return: The updated_at of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_at

    @updated_at.setter
    def updated_at(self, updated_at):
        """Sets the updated_at of this ModelPopulationJoinedFeatureResponseDto.


        :param updated_at: The updated_at of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501
        :type: datetime
        """

        self._updated_at = updated_at

    @property
    def archived_at(self):
        """Gets the archived_at of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501


        :return: The archived_at of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501
        :rtype: datetime
        """
        return self._archived_at

    @archived_at.setter
    def archived_at(self, archived_at):
        """Sets the archived_at of this ModelPopulationJoinedFeatureResponseDto.


        :param archived_at: The archived_at of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501
        :type: datetime
        """

        self._archived_at = archived_at

    @property
    def version(self):
        """Gets the version of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501


        :return: The version of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501
        :rtype: float
        """
        return self._version

    @version.setter
    def version(self, version):
        """Sets the version of this ModelPopulationJoinedFeatureResponseDto.


        :param version: The version of this ModelPopulationJoinedFeatureResponseDto.  # noqa: E501
        :type: float
        """

        self._version = version

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(ModelPopulationJoinedFeatureResponseDto, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ModelPopulationJoinedFeatureResponseDto):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



import click

from curia.infra.databricks.deploy import deploy_databricks_tasks
from curia.infra.databricks.task import DatabricksTaskDefinition
from curia.infra.types import DatabricksTaskDeployResourceNamingConfig
from curia.infra.utils import load_tasks, describe_tasks


@click.group()
def databricks():
    pass


@databricks.command()
@click.argument("project_dir")
@click.argument("package")
@click.option("--verbose", is_flag=True, default=False, help="Toggle verbose output")
def describe(project_dir, package, verbose) -> None:
    """
    Describes all tasks
    """
    tasks = load_tasks(project_dir, package, task_type=DatabricksTaskDefinition)
    describe_tasks(tasks, verbose)


@databricks.command()
@click.argument("project_dir")
@click.argument("package")
@click.argument("prefix")
@click.argument("env")
def deploy(project_dir, package, prefix, env) -> None:
    """
    Deploys all tasks to Databricks, creating the necessary workflows and jobs in the Curia platform and in the
    Databricks workspace.
    """
    tasks = load_tasks(project_dir, package, task_type=DatabricksTaskDefinition)
    if len(tasks) == 0:
        click.echo("No tasks found")
        return
    deploy_databricks_tasks(
        tasks,
        env=env,
        resource_naming_config=DatabricksTaskDeployResourceNamingConfig(
            databricks_workflow_prefix=prefix,
            platform_task_prefix=prefix,
            platform_workflow_prefix=prefix,
        )
    )


if __name__ == "__main__":
    databricks()



import time
from typing import Union, Literal

from curia.api.swagger_client import Model, ModelJob, ModelJobConfig, ContainerConfig
from curia.modeling.exceptions import InvalidModelError, InvalidModelJobError
from curia.modeling.predicted_model_job import PredictedModelJob
from curia.modeling.trained_model_job import TrainedModelJob
from curia.session import Session


class ModelInterface:
    """
    This class is a wrapper for easier model training. Instantiating the class can be done in two ways:

    1) Using an existing session to create a new model: `ModelInterface.with_new_model(...)`
    2) Using an existing session and model: `ModelInterface(session, model)`

    """

    @classmethod
    def with_new_model(cls,
                       session: Session,
                       experiment_name: str,
                       model_type: str,
                       project_id: str,
                       outcome_type: str
                       ):
        """Instantiate class with an existing session and new model.

        Args:
            session (Session): The session object to use.
            experiment_name (str): The name of the experiment
            model_type (str): The type of model. Can be either "risk" or "impactability".
            project_id (str): The project id for the model.
                Navigate to the folder where you want the model to be built, and copy the part of the url that follows https://aledade.curia.ai/projects/ to get the project_id
            outcome_type (str): Can be either "regression" or "occurrence".

        Returns:
            ModelTrainer: The instantiated class.
        """        """"""
        new_model = cls.create_model(experiment_name, model_type, project_id, outcome_type, session)
        return cls(session, new_model)

    def __init__(self, session: Session, model: Model, model_job_id: str = None):
        """Instantiate the model interface

        Args:
            session (Session): Can be instantiated with the `create_session` static method.
            model (Model): Can be instantiated with the `create_model` static method.
            model_job_id (str, optional): The model job id if the model has already been trained. Defaults to None.
        """
        # Set all parameters
        self.session = session
        self.model = model

        # check model is valid
        self.check_model()

        # Set variables for future use
        self.project_id = self.model.project_id
        self.full_exp_name = self.model.name
        self.outcome_type = self.model.outcome_type

        # model_job_id is set when the model is trained
        self.model_job_id = model_job_id

    @staticmethod
    def create_model(experiment_name: str, model_type: str, project_id: str, outcome_type: str,
                     session: Session) -> Model:
        """Create a new model to be used for training.

        Args:
            experiment_name (str): The name of the experiment.
            model_type (str): The type of model. Can be either "risk" or "impactability".
            project_id (str): The folder where the model will be created. Navigate to the folder where you want the model to be built, and copy the part of the url that follows https://aledade.curia.ai/projects/ to get the project_id
            outcome_type (str): Can be either "regression" or "occurrence".
            session (Session): The session object to use. Can be instantiated with `create_session`.

        Returns:
            Model: The new model.
        """
        # Check outcome_type is valid
        if outcome_type not in ["regression", "occurrence"]:
            raise ValueError("outcome_type must be one of: 'regression', 'occurrence'")

        if model_type not in ["risk", "impactability"]:
            raise ValueError("model_type must be one of: 'risk', 'impactability'")

        # Create a new model object
        new_model = Model(
            name=experiment_name,
            type=model_type,
            feature_store='byod',  # bring your own dataset
            project_id=project_id,
            outcome_type=outcome_type  # can be regression or occurrence
        )
        # Register the new model object
        model = session.api_instance.create_one_base_model_controller_model(new_model)

        return model

    def check_model(self):
        """Validates the model.

        Raises:
            InvalidModelError: The model is invalid.
        """
        try:
            self.session.api_instance.get_one_base_model_controller_model(id=self.model.id)
        except Exception as e:
            raise InvalidModelError(f"Model {self.model.id} is invalid: {e}") from e

    def _create_model_job(self,
                          mode: Union[Literal['train'], Literal['predict']],
                          model_job: ModelJob = None,
                          dataset_id: str = None,
                          config: ModelJobConfig = None) -> ModelJob:
        """Helper that creates a model job from config or from an existing model job object.

        Args:
            model_job (ModelJob, optional): A model job object. Defaults to None.
            dataset_id (str, optional): If no model_job passed, then give dataset id. Defaults to None.
            config (ModelJobConfig, optional): Optional additional parameters for creating the model. Defaults to None.

        Returns:
            ModelJob: The model job object that has been fully created.
        """
        if model_job is None:
            # Instantiate a ModelJob object
            new_model_job = ModelJob(
                model_id=self.model.id,
                dataset_id=dataset_id,
                project_id=self.project_id,
                config=config,
                type=mode,
            )
        else:
            new_model_job = model_job

        # create a model job
        new_model_job_to_run = self.session.api_instance.create_one_base_model_job_controller_model_job(
            new_model_job,
        )

        return new_model_job_to_run

    def _update_model_job(self, new_model_job: ModelJob) -> ModelJob:
        """Helper to update a model job.

        Args:
            new_model_job (ModelJob): The new model job object.

        Returns:
            ModelJob: The updated model job object.
        """
        output_model_job = self.session.api_instance.update_one_base_model_job_controller_model_job(
            id=self.model_job_id,
            body=new_model_job,
        )
        return output_model_job

    def _run_model_job(self, model_job_id: str = None) -> str:
        """Helper function to run a model job.

        Args:`
            model_job_id (str, optional): The model job id. Defaults to None.

        Raises:
            InvalidModelJobError: If the model job does not exist.

        Returns:
            str: The model job id.
        """
        # Set the model_job_id if specified in function
        if model_job_id is not None:
            self.model_job_id = model_job_id
        try:
            self.session.api_instance.model_job_controller_start(
                id=self.model_job_id,
                _preload_content=False,
            )
        except Exception as e:
            if "404" in str(e):
                raise InvalidModelJobError(f"Model job {self.model_job_id} does not exist: {e}") from e
            raise e

    def _wait_for_model_job(self, timeout: int, interval: int, print_status: bool) -> ModelJob:
        """Helper function to wait for a model job to finish.

        Args:
            timeout (int): How long to wait for job to complete in seconds.
            interval (int): How often to check in seconds.
            print_status (bool): Whether to print the status of the model job.

        Raises:
            TimeoutError: If the model job does not finish in the given timeout.

        Returns:
            ModelJob: The model job object.
        """
        if print_status:
            print(f"Printing status of model job every {interval} seconds.")

        start_time = time.time()
        while True:
            if time.time() >= timeout + start_time:
                raise TimeoutError(f"Run operation timed out after {timeout} seconds")
            time.sleep(interval)
            model_job = self.session.api_instance.get_one_base_model_job_controller_model_job(id=self.model_job_id)
            status = model_job.status

            if print_status:
                print(f"Model job status: {status}")

            if status not in ["RUNNING", "STARTING"]:
                return model_job

    def train(self,
              model_job: ModelJob = None,
              dataset_id: str = None,
              model_job_config: ModelJobConfig = None,
              wait: bool = False,
              wait_timeout: int = 3600,
              wait_interval: int = 5,
              print_status: bool = True,
              hyperparameter_default_behavior: str = None,
              hyperparameters: dict = None,
              ) -> TrainedModelJob:
        """The train entrypoint for the model interface.

        Args:
            model_job (ModelJob, optional): If a model job object has already been created. Defaults to None.
            dataset_id (str, optional): If no model_job, this is used to create the model_job object. Defaults to None.
            model_job_config (ModelJobConfig, optional): Optional for additional parameters for the model_job. Defaults to None.
            wait (bool, optional): Whether to wait until finished. Defaults to False.
            wait_timeout (int, optional): If wait, then how long to wait in seconds. Defaults to 3600.
            wait_interval (int, optional): If wait, then how often to checkd. Defaults to 5.
            print_status (bool, optional): Whether to print the status of the model job. Defaults to True.
            hyperparameter_default_behavior (str, optional): The default behavior for hyperparameters. Defaults to None.
            hyperparameters (dict, optional): The hyperparameters to use. Defaults to None.

        Returns:
            TrainedModelJob: the TrainedModelJob object.
        """
        # create the model job
        new_model_job_to_run = self._create_model_job('train', model_job, dataset_id, config=model_job_config)
        self.model_job_id = new_model_job_to_run.id

        # Update the new_model_job_to_run with hyperparameters
        if hyperparameters is not None:
            new_model_job_to_run.config = ModelJobConfig(
                container_config=ContainerConfig(
                    hyperparameter_default_behavior=hyperparameter_default_behavior,
                    hyperparameters=hyperparameters,
                )
            )
            self._update_model_job(new_model_job_to_run)

        # run the model job
        self._run_model_job()

        # wait for the model job to finish
        if wait:
            self._wait_for_model_job(wait_timeout, wait_interval, print_status)

        output = TrainedModelJob(self.session, self.model_job_id)
        return output

    def predict(self,
                model_job: ModelJob = None,
                dataset_id: str = None,
                model_job_config: ModelJobConfig = None,
                wait: bool = False,
                wait_timeout: int = 3600,
                wait_interval: int = 5,
                print_status: bool = True,
                ) -> PredictedModelJob:
        """The predict entrypoint for the model interface.

        Args:
            model_job (ModelJob, optional): If a model job object has already been created. Defaults to None.
            dataset_id (str, optional): If no model_job, this is used to create the model_job object. Defaults to None.
            model_job_config (ModelJobConfig, optional): Optional for additional parameters for the model_job. Defaults to None.
            wait (bool, optional): Whether to wait until finished. Defaults to False.
            wait_timeout (int, optional): If wait, then how long to wait in seconds. Defaults to 3600.
            wait_interval (int, optional): If wait, then how often to checkd. Defaults to 5.
            print_status (bool, optional): Whether to print the status of the model job. Defaults to True.

        Returns:
            PredictedModelJob: the PredictedModelJob object.
        """
        # create the model job
        new_model_job_to_run = self._create_model_job('predict', model_job, dataset_id, config=model_job_config)

        # run the model job
        self.model_job_id = new_model_job_to_run.id
        self._run_model_job()

        # wait for the model job to finish
        if wait:
            self._wait_for_model_job(wait_timeout, wait_interval, print_status)

        return PredictedModelJob(self.session, self.model_job_id)



"""
Base class for creating tasks that can be run through the Curia API, with a backend of either a docker container or
a databricks job.
"""
import abc
import dataclasses
import inspect
import typing
import warnings
from typing import List, Callable, Any, Literal, Optional

from curia.infra.types import TaskType
from curia.session import Session
from curia.utils.json import sanitize

TaskIOType = Literal['literal', 'dataset', 'model', 'modelJob']


@dataclasses.dataclass
class TaskInputDefinition:
    """
    Base class for all airml-analytics input definitions, designed to interface with the AirML API
    """
    name: str
    type: TaskIOType
    optional: bool = False
    description: Optional[str] = None

    def describe(self) -> List[str]:
        """
        Describe the input definition
        :return: The description, formatted as a list of strings, with each string representing a line
        """
        return [
            f"Name: {self.name}",
            f"Type: {self.type}",
            f"Optional: {self.optional}",
            f"Description: {self.description}"
        ]


@dataclasses.dataclass
class TaskOutputDefinition:
    """
    Base class for all airml-analytics output definitions, designed to interface with the AirML API
    """
    name: str
    type: TaskIOType
    description: Optional[str] = None

    def describe(self) -> List[str]:
        """
        Describe the output definition
        :return: The description, formatted as a list of strings, with each string representing a line
        """
        return [
            f"Name: {self.name}",
            f"Type: {self.type}",
            f"Description: {self.description}"
        ]


@dataclasses.dataclass
class TaskDefinition(abc.ABC):  # pylint: disable=too-many-instance-attributes
    """
    Base class for creating and managing tasks that can be run through the Curia API, with a backend of either a
    docker container or a databricks job.
    """
    name: str
    task_slug: str
    inputs: List[TaskInputDefinition]
    outputs: List[TaskOutputDefinition]
    description: Optional[str] = None
    # while the function is optional here, it is required for the task to be run. It just gets set in the decorator.
    _function: Optional[Callable] = None  # type: ignore
    _module: Optional[str] = None
    task_type: typing.ClassVar[TaskType] = TaskType.ABSTRACT

    def __call__(self, function: Callable) -> 'TaskDefinition':  # type: ignore
        """
        Decorator to add a function to the analytics task flow definition
        :param function: The function to add
        :return: The function
        """
        self._function = function
        module = inspect.getmodule(function)
        if module is not None:
            self._module = module.__name__
        else:
            self._module = "not_found"
        return self

    @classmethod
    def get_curia_sdk(cls, api_token: str, api_endpoint: str) -> Session:
        """
        Get the Curia SDK using the provided API Key
        :param api_token:
        :param api_endpoint:
        :return:
        """
        return Session(api_token=api_token, host=api_endpoint)

    def verify_task_inputs_valid(self, task_inputs: typing.Dict[str, Any]) -> None:
        """
        Verify that the task inputs are valid by checking that all required inputs are present and that no unexpected
        inputs are present, based on the inputs defined for this task.
        :param task_inputs: The supplied task inputs to verify
        :return:
        """
        errors = []
        for defined_input in self.inputs:
            if not defined_input.optional:
                if defined_input.name not in task_inputs:
                    errors.append(f"Missing required input: {defined_input.name}")
        for actual_input in task_inputs:
            if actual_input in ['taskDefinition', 'notebook_params', 'job_id']:
                # these are special inputs that are not defined in the task definition
                continue
            if actual_input not in [i.name for i in self.inputs]:
                errors.append(f"Unexpected input: {actual_input}")

        if len(errors) > 0:
            errors_str = "\n    ".join(errors)
            raise ValueError(f"Invalid task inputs: \n    {errors_str}")

    def resolve_arguments(self, task_execution_id: str, api_token: str, api_endpoint: str) -> typing.Dict[str, Any]:
        """
        Utilize the Curia SDK to retrieve the arguments for the analytics task flow definition.
        :param task_execution_id: The task execution ID to retrieve the inputs for
        :param api_token: The Platform API token to use
        :param api_endpoint: The Platform API endpoint to use
        :return:
        """
        session = self.get_curia_sdk(api_token, api_endpoint)
        task_execution = session.api_instance.get_one_base_task_execution_controller_task_execution(
            id=task_execution_id
        )
        self.verify_task_inputs_valid(task_execution.inputs)

        resolved_dict = {}
        for task_input in self.inputs:
            if task_input.name not in task_execution.inputs:
                if task_input.optional:
                    continue
                raise ValueError(f"Missing required input: {task_input.name}")
            resolved_dict[task_input.name] = self.resolve_input(
                session,
                task_input,
                task_execution.inputs[task_input.name]
            )
        return resolved_dict

    @classmethod
    def resolve_input(cls,
                      session: Session,  # pylint: disable=unused-argument
                      task_input: TaskInputDefinition,  # pylint: disable=unused-argument
                      task_execution_value: Any) -> Any:
        """
        Resolve a single input, making additional API calls as necessary depending on the task.

        :param session: The Curia SDK session
        :param task_input: The task input definition
        :param task_execution_value: The value of the task input from the Task Execution
        :return: The resolved value
        """
        # this is a stub and will get overridden in specific contexts and environments.
        return task_execution_value

    def upload_results(self,
                       task_execution_id: str,
                       api_token: str,
                       api_endpoint: str,
                       results: typing.Dict[str, Any]) -> None:
        """
        Upload the results of the task's execution to the Curia API and S3
        :param task_execution_id: The task execution ID
        :param api_token: The API key to use to retrieve the task inputs from the Curia API
        :param api_endpoint: The API endpoint to use to retrieve the task inputs from the Curia API
        :param results: The results to upload
        :return: None
        """
        session = self.get_curia_sdk(api_token, api_endpoint)
        session.api_instance.update_one_base_task_execution_controller_task_execution(
            id=task_execution_id,
            body={
                "outputs": sanitize(results)
            }
        )

    def describe(self) -> List[str]:
        """
        Describe the task flow definition
        :return: The description, formatted as a list of strings, with each string representing a line
        """
        assert self._function is not None, "TaskDefinition must decorate a function"
        description = [
            f"Task: {self.name}",
            f"Module: {self.module}",
            f"Function: {self._function.__name__}",
            f"Description: {self.description}",
            f"Task Type: {self.task_type}",
            "Inputs:"
        ]
        for input_def in self.inputs:
            description.extend([
                "    " + line for line in input_def.describe()
            ] + [""])

        description.append("Outputs:")

        for output_def in self.outputs:
            description.extend([
                "    " + line for line in output_def.describe()
            ] + [""])

        return description

    @property
    def module(self) -> str:
        """
        The module that the task flow definition is defined in
        :return:
        """
        return typing.cast(str, self._module)

    @property
    def function_name(self) -> str:
        """
        The name of the function that the task flow definition is defined in
        :return:
        """
        return typing.cast(Callable[[Any], Any], self._function).__name__

    class Registry:
        def __init__(self):
            self._task_definitions: typing.Dict[str, TaskDefinition] = {}

        def register(self, task_definition: 'TaskDefinition') -> None:
            print(f"Registering task {task_definition.name} in module {task_definition._module}")
            if task_definition.name in self._task_definitions:
                if self._task_definitions[task_definition.name]._module != task_definition._module:
                    warnings.warn(
                        f"Task name {task_definition.name} is already registered in module "
                        f"{self._task_definitions[task_definition.name]._module}. "
                        f"Overwriting with task {task_definition.name} in module {task_definition._module}."
                    )
            self._task_definitions[task_definition.name] = task_definition

        def list(self) -> List['TaskDefinition']:
            return list(self._task_definitions.values())

    @abc.abstractmethod
    def run(self,
            task_execution_id: str,
            api_token: str,
            api_endpoint: str,
            **kwargs) -> None:
        """
        Run the analytics task flow definition
        :param task_execution_id: The task execution ID
        :param api_token: The API key to use to retrieve the task inputs from the Curia API
        :param api_endpoint: The API endpoint to use to retrieve the task inputs from the Curia API
        :param kwargs: Additional keyword arguments, specific to ContainerTaskDefinition or DatabricksTaskDefinition
        :return: The result
        """

    @abc.abstractmethod
    def build_workflow_data_block(self, inputs: List[TaskInputDefinition], task_specific_context: dict):
        pass

    @abc.abstractmethod
    def build_task_inputs(self, inputs: List[TaskInputDefinition], task_specific_context: dict):
        pass



# coding: utf-8

"""
    Curia Platform API

    These are the docs for the curia platform API. To test, generate an authorization token first.  # noqa: E501

    OpenAPI spec version: 3.16.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class SelectExpression(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'expression': 'ArithmeticExpression',
        'aggregation': 'str',
        'condition': 'Condition',
        'alias': 'str'
    }

    attribute_map = {
        'expression': 'expression',
        'aggregation': 'aggregation',
        'condition': 'condition',
        'alias': 'alias'
    }

    def __init__(self, expression=None, aggregation=None, condition=None, alias=None):  # noqa: E501
        """SelectExpression - a model defined in Swagger"""  # noqa: E501
        self._expression = None
        self._aggregation = None
        self._condition = None
        self._alias = None
        self.discriminator = None
        self.expression = expression
        self.aggregation = aggregation
        if condition is not None:
            self.condition = condition
        if alias is not None:
            self.alias = alias

    @property
    def expression(self):
        """Gets the expression of this SelectExpression.  # noqa: E501


        :return: The expression of this SelectExpression.  # noqa: E501
        :rtype: ArithmeticExpression
        """
        return self._expression

    @expression.setter
    def expression(self, expression):
        """Sets the expression of this SelectExpression.


        :param expression: The expression of this SelectExpression.  # noqa: E501
        :type: ArithmeticExpression
        """
        if expression is None:
            raise ValueError("Invalid value for `expression`, must not be `None`")  # noqa: E501

        self._expression = expression

    @property
    def aggregation(self):
        """Gets the aggregation of this SelectExpression.  # noqa: E501


        :return: The aggregation of this SelectExpression.  # noqa: E501
        :rtype: str
        """
        return self._aggregation

    @aggregation.setter
    def aggregation(self, aggregation):
        """Sets the aggregation of this SelectExpression.


        :param aggregation: The aggregation of this SelectExpression.  # noqa: E501
        :type: str
        """
        if aggregation is None:
            raise ValueError("Invalid value for `aggregation`, must not be `None`")  # noqa: E501
        allowed_values = ["count", "sum", "avg", "min", "max"]  # noqa: E501
        if aggregation not in allowed_values:
            raise ValueError(
                "Invalid value for `aggregation` ({0}), must be one of {1}"  # noqa: E501
                .format(aggregation, allowed_values)
            )

        self._aggregation = aggregation

    @property
    def condition(self):
        """Gets the condition of this SelectExpression.  # noqa: E501


        :return: The condition of this SelectExpression.  # noqa: E501
        :rtype: Condition
        """
        return self._condition

    @condition.setter
    def condition(self, condition):
        """Sets the condition of this SelectExpression.


        :param condition: The condition of this SelectExpression.  # noqa: E501
        :type: Condition
        """

        self._condition = condition

    @property
    def alias(self):
        """Gets the alias of this SelectExpression.  # noqa: E501


        :return: The alias of this SelectExpression.  # noqa: E501
        :rtype: str
        """
        return self._alias

    @alias.setter
    def alias(self, alias):
        """Sets the alias of this SelectExpression.


        :param alias: The alias of this SelectExpression.  # noqa: E501
        :type: str
        """

        self._alias = alias

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(SelectExpression, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SelectExpression):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



import dataclasses
import datetime
import enum
from typing import Any

import numpy as np
import pandas as pd
from numbers import Integral, Real


def pandas_isnull(obj):
    """
    Helper function to check if an object is null, even if it's a pandas object.
    """
    try:
        return pd.isnull(obj) is True  # prevents issues with pandas arrays treating this as an array
    except ValueError:
        return False


def nonrecursive_dataclass_unpack(dc: Any) -> dict:
    """
    Helper function to unpack a dataclass into a dict, without recursing into other dataclasses. This is useful as we
    want to use custom serialization for the dataclasses (annotating with the dataclass name) but dataclasses.asdict
    will recurse into inner dataclasses and make them indistinguishable from ordinary dicts. This function performs
    the
    """
    return {field.name: getattr(dc, field.name) for field in dataclasses.fields(dc)}


def sanitize(obj, n_digits=5):  # pylint: disable=R0911, R0912
    """
    Sanitizes obj into something json-encodable, and rounds floats to n_digits
    Args:
        obj: Object to sanitize
        n_digits: number of digits to round floats to

    Returns:
        Sanitized version of object.
    """
    if obj is None:
        return "NA"
    if pandas_isnull(obj):
        return "NA"
    if isinstance(obj, Integral):
        return int(obj)
    if isinstance(obj, Real):
        if obj == float("inf"):
            return "inf"
        if obj == float("-inf"):
            return "-inf"
        return round(float(obj), ndigits=n_digits)
    if isinstance(obj, enum.Enum):
        return sanitize(obj.value, n_digits=n_digits)
    if isinstance(obj, np.ndarray):
        return sanitize(obj.tolist(), n_digits=n_digits)
    if isinstance(obj, (bool, bytes, str, int, float)):
        return obj
    if isinstance(obj, list):
        return [sanitize(sub_obj, n_digits=n_digits) for sub_obj in obj]
    if isinstance(obj, tuple):
        return tuple(sanitize(sub_obj, n_digits=n_digits) for sub_obj in obj)
    if isinstance(obj, set):
        return {sanitize(sub_obj, n_digits=n_digits) for sub_obj in obj}
    if isinstance(obj, (datetime.datetime, datetime.date)):
        return obj.isoformat()
    if isinstance(obj, dict):
        obj_dict = obj
        return {sanitize(key, n_digits=n_digits): sanitize(val, n_digits=n_digits) for key, val in obj_dict.items()}
    if dataclasses.is_dataclass(obj):
        return sanitize({'dataclass': type(obj).__name__, **nonrecursive_dataclass_unpack(obj)}, n_digits=n_digits)
    if isinstance(obj, pd.Series):
        return sanitize(list(obj), n_digits=n_digits)
    if isinstance(obj, pd.DataFrame):
        return sanitize(obj.to_dict(orient='records'), n_digits=n_digits)
    raise ValueError(f"Sanitizing value {obj} of type {type(obj)} is not supported. Add code to this function "
                     f"to support sanitization if necessary.")



# coding: utf-8

"""
    Curia Platform API

    These are the docs for the curia platform API. To test, generate an authorization token first.  # noqa: E501

    OpenAPI spec version: 3.16.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class Organization(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'id': 'str',
        'name': 'str',
        'instance_url': 'str',
        'domain': 'str',
        'model_population': 'ModelPopulation',
        'datasets': 'list[Dataset]',
        'projects': 'list[Project]',
        'organization_settings': 'list[OrganizationSetting]',
        'organization_feature_exclusions': 'list[OrganizationFeatureExclusion]',
        'organization_feature_category_exclusions': 'list[OrganizationFeatureCategoryExclusion]',
        'last_updated_by': 'str',
        'created_by': 'str',
        'databases': 'list[Database]',
        'data_queries': 'list[DataQuery]',
        'schedulers': 'list[Scheduler]',
        'workflows': 'list[Workflow]',
        'workflow_templates': 'list[WorkflowTemplate]',
        'workflow_executions': 'list[WorkflowExecution]',
        'data_store': 'list[DataStore]',
        'feature_store': 'list[FeatureStore]',
        'population_store': 'list[PopulationStore]',
        'populations': 'list[Population]',
        'tasks': 'list[Task]',
        'created_at': 'datetime',
        'updated_at': 'datetime',
        'archived_at': 'datetime',
        'version': 'float'
    }

    attribute_map = {
        'id': 'id',
        'name': 'name',
        'instance_url': 'instanceUrl',
        'domain': 'domain',
        'model_population': 'modelPopulation',
        'datasets': 'datasets',
        'projects': 'projects',
        'organization_settings': 'organizationSettings',
        'organization_feature_exclusions': 'organizationFeatureExclusions',
        'organization_feature_category_exclusions': 'organizationFeatureCategoryExclusions',
        'last_updated_by': 'lastUpdatedBy',
        'created_by': 'createdBy',
        'databases': 'databases',
        'data_queries': 'dataQueries',
        'schedulers': 'schedulers',
        'workflows': 'workflows',
        'workflow_templates': 'workflowTemplates',
        'workflow_executions': 'workflowExecutions',
        'data_store': 'dataStore',
        'feature_store': 'featureStore',
        'population_store': 'populationStore',
        'populations': 'populations',
        'tasks': 'tasks',
        'created_at': 'createdAt',
        'updated_at': 'updatedAt',
        'archived_at': 'archivedAt',
        'version': 'version'
    }

    def __init__(self, id=None, name=None, instance_url=None, domain=None, model_population=None, datasets=None, projects=None, organization_settings=None, organization_feature_exclusions=None, organization_feature_category_exclusions=None, last_updated_by=None, created_by=None, databases=None, data_queries=None, schedulers=None, workflows=None, workflow_templates=None, workflow_executions=None, data_store=None, feature_store=None, population_store=None, populations=None, tasks=None, created_at=None, updated_at=None, archived_at=None, version=None):  # noqa: E501
        """Organization - a model defined in Swagger"""  # noqa: E501
        self._id = None
        self._name = None
        self._instance_url = None
        self._domain = None
        self._model_population = None
        self._datasets = None
        self._projects = None
        self._organization_settings = None
        self._organization_feature_exclusions = None
        self._organization_feature_category_exclusions = None
        self._last_updated_by = None
        self._created_by = None
        self._databases = None
        self._data_queries = None
        self._schedulers = None
        self._workflows = None
        self._workflow_templates = None
        self._workflow_executions = None
        self._data_store = None
        self._feature_store = None
        self._population_store = None
        self._populations = None
        self._tasks = None
        self._created_at = None
        self._updated_at = None
        self._archived_at = None
        self._version = None
        self.discriminator = None
        if id is not None:
            self.id = id
        self.name = name
        if instance_url is not None:
            self.instance_url = instance_url
        if domain is not None:
            self.domain = domain
        if model_population is not None:
            self.model_population = model_population
        if datasets is not None:
            self.datasets = datasets
        if projects is not None:
            self.projects = projects
        if organization_settings is not None:
            self.organization_settings = organization_settings
        if organization_feature_exclusions is not None:
            self.organization_feature_exclusions = organization_feature_exclusions
        if organization_feature_category_exclusions is not None:
            self.organization_feature_category_exclusions = organization_feature_category_exclusions
        if last_updated_by is not None:
            self.last_updated_by = last_updated_by
        if created_by is not None:
            self.created_by = created_by
        if databases is not None:
            self.databases = databases
        if data_queries is not None:
            self.data_queries = data_queries
        if schedulers is not None:
            self.schedulers = schedulers
        if workflows is not None:
            self.workflows = workflows
        if workflow_templates is not None:
            self.workflow_templates = workflow_templates
        if workflow_executions is not None:
            self.workflow_executions = workflow_executions
        if data_store is not None:
            self.data_store = data_store
        if feature_store is not None:
            self.feature_store = feature_store
        if population_store is not None:
            self.population_store = population_store
        if populations is not None:
            self.populations = populations
        if tasks is not None:
            self.tasks = tasks
        if created_at is not None:
            self.created_at = created_at
        if updated_at is not None:
            self.updated_at = updated_at
        if archived_at is not None:
            self.archived_at = archived_at
        if version is not None:
            self.version = version

    @property
    def id(self):
        """Gets the id of this Organization.  # noqa: E501


        :return: The id of this Organization.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this Organization.


        :param id: The id of this Organization.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def name(self):
        """Gets the name of this Organization.  # noqa: E501


        :return: The name of this Organization.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this Organization.


        :param name: The name of this Organization.  # noqa: E501
        :type: str
        """
        if name is None:
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def instance_url(self):
        """Gets the instance_url of this Organization.  # noqa: E501


        :return: The instance_url of this Organization.  # noqa: E501
        :rtype: str
        """
        return self._instance_url

    @instance_url.setter
    def instance_url(self, instance_url):
        """Sets the instance_url of this Organization.


        :param instance_url: The instance_url of this Organization.  # noqa: E501
        :type: str
        """

        self._instance_url = instance_url

    @property
    def domain(self):
        """Gets the domain of this Organization.  # noqa: E501


        :return: The domain of this Organization.  # noqa: E501
        :rtype: str
        """
        return self._domain

    @domain.setter
    def domain(self, domain):
        """Sets the domain of this Organization.


        :param domain: The domain of this Organization.  # noqa: E501
        :type: str
        """

        self._domain = domain

    @property
    def model_population(self):
        """Gets the model_population of this Organization.  # noqa: E501


        :return: The model_population of this Organization.  # noqa: E501
        :rtype: ModelPopulation
        """
        return self._model_population

    @model_population.setter
    def model_population(self, model_population):
        """Sets the model_population of this Organization.


        :param model_population: The model_population of this Organization.  # noqa: E501
        :type: ModelPopulation
        """

        self._model_population = model_population

    @property
    def datasets(self):
        """Gets the datasets of this Organization.  # noqa: E501


        :return: The datasets of this Organization.  # noqa: E501
        :rtype: list[Dataset]
        """
        return self._datasets

    @datasets.setter
    def datasets(self, datasets):
        """Sets the datasets of this Organization.


        :param datasets: The datasets of this Organization.  # noqa: E501
        :type: list[Dataset]
        """

        self._datasets = datasets

    @property
    def projects(self):
        """Gets the projects of this Organization.  # noqa: E501


        :return: The projects of this Organization.  # noqa: E501
        :rtype: list[Project]
        """
        return self._projects

    @projects.setter
    def projects(self, projects):
        """Sets the projects of this Organization.


        :param projects: The projects of this Organization.  # noqa: E501
        :type: list[Project]
        """

        self._projects = projects

    @property
    def organization_settings(self):
        """Gets the organization_settings of this Organization.  # noqa: E501


        :return: The organization_settings of this Organization.  # noqa: E501
        :rtype: list[OrganizationSetting]
        """
        return self._organization_settings

    @organization_settings.setter
    def organization_settings(self, organization_settings):
        """Sets the organization_settings of this Organization.


        :param organization_settings: The organization_settings of this Organization.  # noqa: E501
        :type: list[OrganizationSetting]
        """

        self._organization_settings = organization_settings

    @property
    def organization_feature_exclusions(self):
        """Gets the organization_feature_exclusions of this Organization.  # noqa: E501


        :return: The organization_feature_exclusions of this Organization.  # noqa: E501
        :rtype: list[OrganizationFeatureExclusion]
        """
        return self._organization_feature_exclusions

    @organization_feature_exclusions.setter
    def organization_feature_exclusions(self, organization_feature_exclusions):
        """Sets the organization_feature_exclusions of this Organization.


        :param organization_feature_exclusions: The organization_feature_exclusions of this Organization.  # noqa: E501
        :type: list[OrganizationFeatureExclusion]
        """

        self._organization_feature_exclusions = organization_feature_exclusions

    @property
    def organization_feature_category_exclusions(self):
        """Gets the organization_feature_category_exclusions of this Organization.  # noqa: E501


        :return: The organization_feature_category_exclusions of this Organization.  # noqa: E501
        :rtype: list[OrganizationFeatureCategoryExclusion]
        """
        return self._organization_feature_category_exclusions

    @organization_feature_category_exclusions.setter
    def organization_feature_category_exclusions(self, organization_feature_category_exclusions):
        """Sets the organization_feature_category_exclusions of this Organization.


        :param organization_feature_category_exclusions: The organization_feature_category_exclusions of this Organization.  # noqa: E501
        :type: list[OrganizationFeatureCategoryExclusion]
        """

        self._organization_feature_category_exclusions = organization_feature_category_exclusions

    @property
    def last_updated_by(self):
        """Gets the last_updated_by of this Organization.  # noqa: E501


        :return: The last_updated_by of this Organization.  # noqa: E501
        :rtype: str
        """
        return self._last_updated_by

    @last_updated_by.setter
    def last_updated_by(self, last_updated_by):
        """Sets the last_updated_by of this Organization.


        :param last_updated_by: The last_updated_by of this Organization.  # noqa: E501
        :type: str
        """

        self._last_updated_by = last_updated_by

    @property
    def created_by(self):
        """Gets the created_by of this Organization.  # noqa: E501


        :return: The created_by of this Organization.  # noqa: E501
        :rtype: str
        """
        return self._created_by

    @created_by.setter
    def created_by(self, created_by):
        """Sets the created_by of this Organization.


        :param created_by: The created_by of this Organization.  # noqa: E501
        :type: str
        """

        self._created_by = created_by

    @property
    def databases(self):
        """Gets the databases of this Organization.  # noqa: E501


        :return: The databases of this Organization.  # noqa: E501
        :rtype: list[Database]
        """
        return self._databases

    @databases.setter
    def databases(self, databases):
        """Sets the databases of this Organization.


        :param databases: The databases of this Organization.  # noqa: E501
        :type: list[Database]
        """

        self._databases = databases

    @property
    def data_queries(self):
        """Gets the data_queries of this Organization.  # noqa: E501


        :return: The data_queries of this Organization.  # noqa: E501
        :rtype: list[DataQuery]
        """
        return self._data_queries

    @data_queries.setter
    def data_queries(self, data_queries):
        """Sets the data_queries of this Organization.


        :param data_queries: The data_queries of this Organization.  # noqa: E501
        :type: list[DataQuery]
        """

        self._data_queries = data_queries

    @property
    def schedulers(self):
        """Gets the schedulers of this Organization.  # noqa: E501


        :return: The schedulers of this Organization.  # noqa: E501
        :rtype: list[Scheduler]
        """
        return self._schedulers

    @schedulers.setter
    def schedulers(self, schedulers):
        """Sets the schedulers of this Organization.


        :param schedulers: The schedulers of this Organization.  # noqa: E501
        :type: list[Scheduler]
        """

        self._schedulers = schedulers

    @property
    def workflows(self):
        """Gets the workflows of this Organization.  # noqa: E501


        :return: The workflows of this Organization.  # noqa: E501
        :rtype: list[Workflow]
        """
        return self._workflows

    @workflows.setter
    def workflows(self, workflows):
        """Sets the workflows of this Organization.


        :param workflows: The workflows of this Organization.  # noqa: E501
        :type: list[Workflow]
        """

        self._workflows = workflows

    @property
    def workflow_templates(self):
        """Gets the workflow_templates of this Organization.  # noqa: E501


        :return: The workflow_templates of this Organization.  # noqa: E501
        :rtype: list[WorkflowTemplate]
        """
        return self._workflow_templates

    @workflow_templates.setter
    def workflow_templates(self, workflow_templates):
        """Sets the workflow_templates of this Organization.


        :param workflow_templates: The workflow_templates of this Organization.  # noqa: E501
        :type: list[WorkflowTemplate]
        """

        self._workflow_templates = workflow_templates

    @property
    def workflow_executions(self):
        """Gets the workflow_executions of this Organization.  # noqa: E501


        :return: The workflow_executions of this Organization.  # noqa: E501
        :rtype: list[WorkflowExecution]
        """
        return self._workflow_executions

    @workflow_executions.setter
    def workflow_executions(self, workflow_executions):
        """Sets the workflow_executions of this Organization.


        :param workflow_executions: The workflow_executions of this Organization.  # noqa: E501
        :type: list[WorkflowExecution]
        """

        self._workflow_executions = workflow_executions

    @property
    def data_store(self):
        """Gets the data_store of this Organization.  # noqa: E501


        :return: The data_store of this Organization.  # noqa: E501
        :rtype: list[DataStore]
        """
        return self._data_store

    @data_store.setter
    def data_store(self, data_store):
        """Sets the data_store of this Organization.


        :param data_store: The data_store of this Organization.  # noqa: E501
        :type: list[DataStore]
        """

        self._data_store = data_store

    @property
    def feature_store(self):
        """Gets the feature_store of this Organization.  # noqa: E501


        :return: The feature_store of this Organization.  # noqa: E501
        :rtype: list[FeatureStore]
        """
        return self._feature_store

    @feature_store.setter
    def feature_store(self, feature_store):
        """Sets the feature_store of this Organization.


        :param feature_store: The feature_store of this Organization.  # noqa: E501
        :type: list[FeatureStore]
        """

        self._feature_store = feature_store

    @property
    def population_store(self):
        """Gets the population_store of this Organization.  # noqa: E501


        :return: The population_store of this Organization.  # noqa: E501
        :rtype: list[PopulationStore]
        """
        return self._population_store

    @population_store.setter
    def population_store(self, population_store):
        """Sets the population_store of this Organization.


        :param population_store: The population_store of this Organization.  # noqa: E501
        :type: list[PopulationStore]
        """

        self._population_store = population_store

    @property
    def populations(self):
        """Gets the populations of this Organization.  # noqa: E501


        :return: The populations of this Organization.  # noqa: E501
        :rtype: list[Population]
        """
        return self._populations

    @populations.setter
    def populations(self, populations):
        """Sets the populations of this Organization.


        :param populations: The populations of this Organization.  # noqa: E501
        :type: list[Population]
        """

        self._populations = populations

    @property
    def tasks(self):
        """Gets the tasks of this Organization.  # noqa: E501


        :return: The tasks of this Organization.  # noqa: E501
        :rtype: list[Task]
        """
        return self._tasks

    @tasks.setter
    def tasks(self, tasks):
        """Sets the tasks of this Organization.


        :param tasks: The tasks of this Organization.  # noqa: E501
        :type: list[Task]
        """

        self._tasks = tasks

    @property
    def created_at(self):
        """Gets the created_at of this Organization.  # noqa: E501


        :return: The created_at of this Organization.  # noqa: E501
        :rtype: datetime
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at):
        """Sets the created_at of this Organization.


        :param created_at: The created_at of this Organization.  # noqa: E501
        :type: datetime
        """

        self._created_at = created_at

    @property
    def updated_at(self):
        """Gets the updated_at of this Organization.  # noqa: E501


        :return: The updated_at of this Organization.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_at

    @updated_at.setter
    def updated_at(self, updated_at):
        """Sets the updated_at of this Organization.


        :param updated_at: The updated_at of this Organization.  # noqa: E501
        :type: datetime
        """

        self._updated_at = updated_at

    @property
    def archived_at(self):
        """Gets the archived_at of this Organization.  # noqa: E501


        :return: The archived_at of this Organization.  # noqa: E501
        :rtype: datetime
        """
        return self._archived_at

    @archived_at.setter
    def archived_at(self, archived_at):
        """Sets the archived_at of this Organization.


        :param archived_at: The archived_at of this Organization.  # noqa: E501
        :type: datetime
        """

        self._archived_at = archived_at

    @property
    def version(self):
        """Gets the version of this Organization.  # noqa: E501


        :return: The version of this Organization.  # noqa: E501
        :rtype: float
        """
        return self._version

    @version.setter
    def version(self, version):
        """Sets the version of this Organization.


        :param version: The version of this Organization.  # noqa: E501
        :type: float
        """

        self._version = version

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(Organization, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Organization):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



import dataclasses
from typing import Optional, ClassVar, List

from curia.infra.task import TaskDefinition, TaskInputDefinition
from curia.infra.types import TaskType
from curia.utils.s3 import AWSIoContext, AWSConfig


@dataclasses.dataclass
class ContainerTaskDefinition(TaskDefinition):  # pylint: disable=too-many-instance-attributes
    """
    Task definition for a task executed through a container
    """
    task_type: ClassVar[TaskType] = TaskType.CONTAINER

    # pylint: disable=arguments-differ
    def run(self,
            task_execution_id: str,
            api_token: str,
            api_endpoint: str,
            aws_config: Optional[AWSConfig] = None) -> None:
        """
        Run the analytics task flow definition
        :param task_execution_id: The task execution ID
        :param api_token: The API key to use to retrieve the task inputs from the Curia API
        :param api_endpoint: The API endpoint to use to retrieve the task inputs from the Curia API
        :param aws_config: The AWS region to use for S3
        :return: The result
        """
        if aws_config is None:
            aws_config = AWSConfig()

        with AWSIoContext(
                **aws_config.to_kwargs()
        ):
            assert self._function is not None, "TaskDefinition must decorate a function"
            resolved_args = self.resolve_arguments(task_execution_id, api_token, api_endpoint)
            results = self._function(**resolved_args)
            self.upload_results(task_execution_id, api_token, api_endpoint, results)

    def build_workflow_data_block(self, inputs: List[TaskInputDefinition], task_specific_context: dict):
        if "ecs_task_family" not in task_specific_context:
            raise ValueError("ECS task family must be provided to ContainerTaskDefinition")
        return {
            **{task_input.name: "{{parameters." + task_input.name + "}}" for task_input in inputs},
            "taskDefinition": task_specific_context["ecs_task_family"]
        }

    def build_task_inputs(self, inputs: List[TaskInputDefinition], task_specific_context: dict):
        if "ecs_task_family" not in task_specific_context:
            raise ValueError("ECS task family must be provided to ContainerTaskDefinition")
        return {
            **{
                task_input.name: {
                    "required": not task_input.optional,
                    "descr": task_input.description
                } for task_input in inputs
            },
            "taskDefinition": {
                "default": task_specific_context["ecs_task_family"],
                "required": False
            }
        }



# coding: utf-8

"""
    Curia Platform API

    These are the docs for the curia platform API. To test, generate an authorization token first.  # noqa: E501

    OpenAPI spec version: 3.16.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class BooleanExpression(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'enabled': 'bool',
        'logic': 'str',
        'expressions': 'list[BooleanExpression]',
        'field': 'str',
        'operator': 'object',
        'value': 'object'
    }

    attribute_map = {
        'enabled': 'enabled',
        'logic': 'logic',
        'expressions': 'expressions',
        'field': 'field',
        'operator': 'operator',
        'value': 'value'
    }

    def __init__(self, enabled=None, logic=None, expressions=None, field=None, operator=None, value=None):  # noqa: E501
        """BooleanExpression - a model defined in Swagger"""  # noqa: E501
        self._enabled = None
        self._logic = None
        self._expressions = None
        self._field = None
        self._operator = None
        self._value = None
        self.discriminator = None
        if enabled is not None:
            self.enabled = enabled
        if logic is not None:
            self.logic = logic
        if expressions is not None:
            self.expressions = expressions
        if field is not None:
            self.field = field
        if operator is not None:
            self.operator = operator
        if value is not None:
            self.value = value

    @property
    def enabled(self):
        """Gets the enabled of this BooleanExpression.  # noqa: E501


        :return: The enabled of this BooleanExpression.  # noqa: E501
        :rtype: bool
        """
        return self._enabled

    @enabled.setter
    def enabled(self, enabled):
        """Sets the enabled of this BooleanExpression.


        :param enabled: The enabled of this BooleanExpression.  # noqa: E501
        :type: bool
        """

        self._enabled = enabled

    @property
    def logic(self):
        """Gets the logic of this BooleanExpression.  # noqa: E501


        :return: The logic of this BooleanExpression.  # noqa: E501
        :rtype: str
        """
        return self._logic

    @logic.setter
    def logic(self, logic):
        """Sets the logic of this BooleanExpression.


        :param logic: The logic of this BooleanExpression.  # noqa: E501
        :type: str
        """
        allowed_values = ["and", "or", "not"]  # noqa: E501
        if logic not in allowed_values:
            raise ValueError(
                "Invalid value for `logic` ({0}), must be one of {1}"  # noqa: E501
                .format(logic, allowed_values)
            )

        self._logic = logic

    @property
    def expressions(self):
        """Gets the expressions of this BooleanExpression.  # noqa: E501


        :return: The expressions of this BooleanExpression.  # noqa: E501
        :rtype: list[BooleanExpression]
        """
        return self._expressions

    @expressions.setter
    def expressions(self, expressions):
        """Sets the expressions of this BooleanExpression.


        :param expressions: The expressions of this BooleanExpression.  # noqa: E501
        :type: list[BooleanExpression]
        """

        self._expressions = expressions

    @property
    def field(self):
        """Gets the field of this BooleanExpression.  # noqa: E501


        :return: The field of this BooleanExpression.  # noqa: E501
        :rtype: str
        """
        return self._field

    @field.setter
    def field(self, field):
        """Sets the field of this BooleanExpression.


        :param field: The field of this BooleanExpression.  # noqa: E501
        :type: str
        """

        self._field = field

    @property
    def operator(self):
        """Gets the operator of this BooleanExpression.  # noqa: E501


        :return: The operator of this BooleanExpression.  # noqa: E501
        :rtype: object
        """
        return self._operator

    @operator.setter
    def operator(self, operator):
        """Sets the operator of this BooleanExpression.


        :param operator: The operator of this BooleanExpression.  # noqa: E501
        :type: object
        """

        self._operator = operator

    @property
    def value(self):
        """Gets the value of this BooleanExpression.  # noqa: E501


        :return: The value of this BooleanExpression.  # noqa: E501
        :rtype: object
        """
        return self._value

    @value.setter
    def value(self, value):
        """Sets the value of this BooleanExpression.


        :param value: The value of this BooleanExpression.  # noqa: E501
        :type: object
        """

        self._value = value

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(BooleanExpression, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, BooleanExpression):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Curia Platform API

    These are the docs for the curia platform API. To test, generate an authorization token first.  # noqa: E501

    OpenAPI spec version: 3.16.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class GetManyUserAuditTrailResponseDto(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'data': 'list[UserAuditTrailResponseDto]',
        'count': 'float',
        'total': 'float',
        'page': 'float',
        'page_count': 'float'
    }

    attribute_map = {
        'data': 'data',
        'count': 'count',
        'total': 'total',
        'page': 'page',
        'page_count': 'pageCount'
    }

    def __init__(self, data=None, count=None, total=None, page=None, page_count=None):  # noqa: E501
        """GetManyUserAuditTrailResponseDto - a model defined in Swagger"""  # noqa: E501
        self._data = None
        self._count = None
        self._total = None
        self._page = None
        self._page_count = None
        self.discriminator = None
        self.data = data
        self.count = count
        self.total = total
        self.page = page
        self.page_count = page_count

    @property
    def data(self):
        """Gets the data of this GetManyUserAuditTrailResponseDto.  # noqa: E501


        :return: The data of this GetManyUserAuditTrailResponseDto.  # noqa: E501
        :rtype: list[UserAuditTrailResponseDto]
        """
        return self._data

    @data.setter
    def data(self, data):
        """Sets the data of this GetManyUserAuditTrailResponseDto.


        :param data: The data of this GetManyUserAuditTrailResponseDto.  # noqa: E501
        :type: list[UserAuditTrailResponseDto]
        """
        if data is None:
            raise ValueError("Invalid value for `data`, must not be `None`")  # noqa: E501

        self._data = data

    @property
    def count(self):
        """Gets the count of this GetManyUserAuditTrailResponseDto.  # noqa: E501


        :return: The count of this GetManyUserAuditTrailResponseDto.  # noqa: E501
        :rtype: float
        """
        return self._count

    @count.setter
    def count(self, count):
        """Sets the count of this GetManyUserAuditTrailResponseDto.


        :param count: The count of this GetManyUserAuditTrailResponseDto.  # noqa: E501
        :type: float
        """
        if count is None:
            raise ValueError("Invalid value for `count`, must not be `None`")  # noqa: E501

        self._count = count

    @property
    def total(self):
        """Gets the total of this GetManyUserAuditTrailResponseDto.  # noqa: E501


        :return: The total of this GetManyUserAuditTrailResponseDto.  # noqa: E501
        :rtype: float
        """
        return self._total

    @total.setter
    def total(self, total):
        """Sets the total of this GetManyUserAuditTrailResponseDto.


        :param total: The total of this GetManyUserAuditTrailResponseDto.  # noqa: E501
        :type: float
        """
        if total is None:
            raise ValueError("Invalid value for `total`, must not be `None`")  # noqa: E501

        self._total = total

    @property
    def page(self):
        """Gets the page of this GetManyUserAuditTrailResponseDto.  # noqa: E501


        :return: The page of this GetManyUserAuditTrailResponseDto.  # noqa: E501
        :rtype: float
        """
        return self._page

    @page.setter
    def page(self, page):
        """Sets the page of this GetManyUserAuditTrailResponseDto.


        :param page: The page of this GetManyUserAuditTrailResponseDto.  # noqa: E501
        :type: float
        """
        if page is None:
            raise ValueError("Invalid value for `page`, must not be `None`")  # noqa: E501

        self._page = page

    @property
    def page_count(self):
        """Gets the page_count of this GetManyUserAuditTrailResponseDto.  # noqa: E501


        :return: The page_count of this GetManyUserAuditTrailResponseDto.  # noqa: E501
        :rtype: float
        """
        return self._page_count

    @page_count.setter
    def page_count(self, page_count):
        """Sets the page_count of this GetManyUserAuditTrailResponseDto.


        :param page_count: The page_count of this GetManyUserAuditTrailResponseDto.  # noqa: E501
        :type: float
        """
        if page_count is None:
            raise ValueError("Invalid value for `page_count`, must not be `None`")  # noqa: E501

        self._page_count = page_count

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(GetManyUserAuditTrailResponseDto, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, GetManyUserAuditTrailResponseDto):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



import datetime
import inspect
import re
import warnings
from typing import Callable, TypeVar, Any, Optional, Type, List, Dict
from unittest.mock import MagicMock

SwaggerObj = TypeVar("SwaggerObj", bound="swagger_mock")


def _get_attr_name_simple_check(line: str) -> Optional[str]:
    """
    line = if not patient_metadata_filters:
    """
    match = re.search(r'if (\w+) not in allowed_values:', line)
    if match:
        return match.group(1)
    return None


def _get_attr_name_set_check(line: str) -> Optional[str]:
    """
    line = if not set(patient_metadata_filters).issubset(set(allowed_values)):
    """
    match = re.search(r'if not set\((\w+)\).issubset\(set\(allowed_values\)\):', line)
    if match:
        return match.group(1)
    return None


def _get_allowed_values(cls: Type) -> Optional[Dict[str, List[Any]]]:
    """
    Match blocks like and extract allowed values from them

    allowed_values = ["train", "predict"]  # noqa: E501
    if <attr> not in allowed_values:
    """
    try:
        source = inspect.getsource(cls)
        attr_to_allowed_values = {}
        source_lines = source.split('\n')
        while source_lines:
            line = source_lines.pop(0)
            if 'allowed_values = ' in line:
                post_assignment = '='.join(line.split('=')[1:])
                to_evaluate = re.sub(r'#[^\'"]*$', '', post_assignment).strip()
                # pylint: disable=eval-used
                allowed_values = eval(to_evaluate)
                attr_check_line = source_lines.pop(0)
                if attr := _get_attr_name_simple_check(attr_check_line):
                    attr_to_allowed_values[attr] = allowed_values
                elif attr := _get_attr_name_set_check(attr_check_line):
                    attr_to_allowed_values[attr] = [allowed_values]
        return attr_to_allowed_values
    except TypeError:
        return None
    except OSError:
        return None


def _get_mock_value(argument_type: str, key: str, allowed_values: List[Any]) -> Any:
    # pylint: disable=too-many-return-statements
    if allowed_values:
        return allowed_values[0]
    if argument_type == 'str':
        return f'{key}_value'
    if argument_type == 'int':
        return 1
    if argument_type == 'float':
        return 1.0
    if argument_type == 'bool':
        return True
    if argument_type == 'datetime':
        return datetime.datetime.now()
    if argument_type == 'list':
        return []
    if argument_type == 'dict':
        return {}
    return MagicMock()


def swagger_mock(cls: Type[SwaggerObj], strict=False) -> Callable[..., SwaggerObj]:
    assert hasattr(cls, 'swagger_types'), \
        f'{cls} does not have swagger_types and must not be an autogenerated swagger object'
    swagger_types = cls.swagger_types
    allowed_values = _get_allowed_values(cls) or {}

    def constructor(**kwargs: Dict[str, Any]) -> SwaggerObj:
        for key, argument_type in swagger_types.items():
            if key not in kwargs:
                kwargs[key] = _get_mock_value(argument_type, key, allowed_values.get(key, None))
        try:
            return cls(**kwargs)
        except ValueError as e:
            if strict:
                raise e
            warnings.warn(f'Error creating {cls} with {kwargs}: {e}\n'
                          f'Creating a MagicMock instead. Unforeseen consequences may occur.')
            return MagicMock(spec=cls)
    return constructor



from curia.session import Session
import pandas as pd
import numpy as np
from typing import Dict, List, Callable

from curia.api.swagger_client.models.model_job_output_response_dto import ModelJobOutputResponseDto

class MetricsInterface:
    metric_factories = dict()
    """
    A class to simplify the model job output extraction process.

    Much of this code was originally in:
    https://dbc-5c784d6d-d7a7.cloud.databricks.com/?o=1189584077923919#notebook/3744383587579554/command/3744383587579629
    """

    @classmethod
    def _get_raw_outputs(cls, session: Session, model_job_id: str) -> List[ModelJobOutputResponseDto]:
        """Query the API for all outputs of a model job.

        Args:
            session (Session): The session object to use.
            model_job_id (str): The model job from which to get the outputs.

        Returns:
            List[ModelJobOutputResponseDto]: The full list of outputs from the model job.
        """
        raw_outputs = session.api_instance.get_many_base_model_job_output_controller_model_job_output(
            filter=[f"modelJobId||$eq||{model_job_id}"]  # gt, lt, neq
        ).data
        return raw_outputs

    @classmethod
    def _get_named_output_with_session(cls,
                          session: Session,
                          output_name: str,
                          model_job_id: str,
                          ) -> ModelJobOutputResponseDto:
        # fetch raw outputs if not provided

        raw_outputs = cls._get_raw_outputs(session, model_job_id)
        # get named output
        return cls._get_named_output(raw_outputs, output_name)

    @classmethod
    def get_named_output_with_session(cls,
                          session: Session,
                          output_name: str,
                          model_job_id: str,
                          ) -> ModelJobOutputResponseDto:
        """Fetch a named output from a model job output.

        Args:
            session (Session): The session object to use.
            output_name (str): The name of the output to fetch.
            model_job_id (str): The model job from which to get the output.

        Returns:
            ModelJobOutputResponseDto: The named output.
        """
        return cls._get_named_output_with_session(session, output_name, model_job_id)

    @classmethod
    def _get_named_output(cls, raw_outputs: List[ModelJobOutputResponseDto], output_name: str) -> ModelJobOutputResponseDto:
        try:
            named_output = [output for output in raw_outputs if output.name == output_name][0]
        except IndexError as e:
            raise ValueError(f"No output with name {output_name} found.") from e

        return named_output


    @classmethod
    def get_basic_metrics(cls, session: Session, model_job_id: str) -> Dict[str, float or int]:
        """Extracting the most basic metrics from the model job as a dictionary.

        Args:
            session (Session): The session object to use.
            model_job_id (str): The model job id from which to get the metrics

        Returns:
            Dict[str, float or int]: _description_
        """
        model_summary = cls._get_named_output_with_session(session, "model-summary", model_job_id)
        metrics = model_summary.data["metrics"]
        return metrics

    @classmethod
    def get_registered_metrics(cls) -> List[str]:
        """Get the names of metrics that can be used with the get_metrics function.

        Returns:
            List[str]: The list of metric names.
        """
        return list(cls.metric_factories.keys())

    @classmethod
    def _remove_na(cls, dirty_output: pd.DataFrame, replacement: int = 0):
        """Helper function to remove NA and np.nan from a dictionary.

        Args:
            dirty_output (pd.DataFrame): The dataframe with dirty values.
            replacement (int, optional): What value to put in the cleaned dictionary. Defaults to 0.

        """
        to_replace = [None, np.nan, "NA"]
        dirty_output.replace(to_replace, replacement, inplace=True)

    @classmethod
    def get_metrics(cls,
                    session: Session,
                    metric_names: List[str],
                    model_job_id: str) -> Dict[str, float or int]:
        """Get the metrics named in 'metric_names' from the model job.

        Args:
            session (Session): The session object to use.
            metric_names (List[str]): The metrics to pull.
            model_job_id (str): The model job from which to get output.

        Raises:
            ValueError: If any metric is not a registered metric

        Returns:
            Dict[str, float or int]: The metrics requested.
        """
        raw_outputs = cls._get_raw_outputs(session, model_job_id)
        # Check that all metrics are registered
        registered_metric_names = cls.get_registered_metrics()
        for metric in metric_names:
            if metric not in registered_metric_names:
                raise ValueError(f"Metric {metric} is not registered. Registered metrics are: {registered_metric_names}")

        # calculate the metrics and add them to the output
        output = dict()
        for metric_name in metric_names:
            output[metric_name] = cls.metric_factories[metric_name](raw_outputs)

        return output

def register_metric(metric_name: str):
    """Register a metric with the instance.

    Args:
        metric_name (str): The name of the metric.
    """
    def decorator(concrete_method: Callable[[List[ModelJobOutputResponseDto]], float or pd.DataFrame or bool or int]):
        MetricsInterface.metric_factories[metric_name] = concrete_method
        return concrete_method
    return decorator


@register_metric("auc")
def _get_auc(raw_outputs: List[ModelJobOutputResponseDto]) -> float:
    """
    Get the auc_roc from the model outputs for 'occurrence' models.
    """
    model_summary = MetricsInterface._get_named_output(raw_outputs, "model-summary")
    output = model_summary.data["metrics"]["auc_roc"]
    return output

@register_metric("r2")
def _get_r2(raw_outputs: List[ModelJobOutputResponseDto]) -> float:
    """
    Get the r2 from the model outputs for 'regression' models.
    """
    model_summary = MetricsInterface._get_named_output(raw_outputs, "model-summary")
    output = model_summary.data["metrics"]["r2"]
    return output

@register_metric("smd")
def _get_smd(raw_outputs: List[ModelJobOutputResponseDto]) -> pd.DataFrame:
    """
    Get the smd from model outputs.
    """
    smd_features = MetricsInterface._get_named_output(raw_outputs, "top-features")
    output = smd_features.data
    output_df = pd.DataFrame(output)
    MetricsInterface._remove_na(output_df)
    return output_df

@register_metric("shap")
def _get_shap(raw_outputs: List[ModelJobOutputResponseDto]) -> pd.DataFrame:
    """
    Gets shap values from model outputs.
    """
    shap_features = MetricsInterface._get_named_output(raw_outputs, "shap-features")
    output = shap_features.data
    output_df = pd.DataFrame(output)
    # Transpose and rename index -> feature for consistency with smd
    output_df = output_df.T.reset_index()
    output_df.rename(columns={"index": "feature"}, inplace=True)
    MetricsInterface._remove_na(output_df)
    return output_df

@register_metric("decile_gain")
def _get_decile_gain(raw_outputs: List[ModelJobOutputResponseDto]) -> pd.DataFrame:
    """
    Get the decile gain from model outputs.
    """
    decile_gain = MetricsInterface._get_named_output(raw_outputs, "bin-summary")
    output = decile_gain.data
    output_df = pd.DataFrame(output)
    MetricsInterface._remove_na(output_df)
    return output_df
